@@ -11,9 +11,9 @@ from .. import text, util, exception
 from ..cache import memcache
 
 BASE_PATTERN = r"(?:https?://)?(?:[\w-]+\.)?facebook\.com"
-USER_PATTERN = (BASE_PATTERN +
-                r"/(?!media/|photo/|photo.php|watch/)"
-                r"(?:profile\.php\?id=|people/[^/?#]+/)?([^/?&#]+)")
+USER_PATTERN = (rf"{BASE_PATTERN}/"
+                rf"(?!media/|photo/|photo.php|watch/|permalink.php)"
+                rf"(?:profile\.php\?id=|people/[^/?#]+/)?([^/?&#]+)")
 
 
 class FacebookExtractor(Extractor):
@@ -108,7 +108,7 @@ class FacebookExtractor(Extractor):
                 '"message":{"delight_ranges"',
                 '"},"message_preferred_body"'
             ).rsplit('],"text":"', 1)[-1]),
-            "date": text.parse_timestamp(
+            "date": self.parse_timestamp(
                 text.extr(photo_page, '\\"publish_time\\":', ',') or
                 text.extr(photo_page, '"created_time":', ',')
             ),
@@ -172,7 +172,7 @@ class FacebookExtractor(Extractor):
             "user_id": text.extr(
                 video_page, '"owner":{"__typename":"User","id":"', '"'
             ),
-            "date": text.parse_timestamp(text.extr(
+            "date": self.parse_timestamp(text.extr(
                 video_page, '\\"publish_time\\":', ','
             )),
             "type": "video"
@@ -292,7 +292,7 @@ class FacebookExtractor(Extractor):
             else:
                 retries = 0
                 photo.update(set_data)
-                yield Message.Directory, photo
+                yield Message.Directory, "", photo
                 yield Message.Url, photo["url"], photo
 
             if not photo["next_photo_id"]:
@@ -369,6 +369,16 @@ class FacebookExtractor(Extractor):
                 for edge in (user["profile_tabs"]["profile_user"]
                              ["timeline_nav_app_sections"]["edges"])
             ]
+
+            if bio := text.extr(page, '"best_description":{"text":"', '"'):
+                user["biography"] = self.decode_all(bio)
+            elif (pos := page.find(
+                    '"__module_operation_ProfileCometTileView_profileT')) >= 0:
+                user["biography"] = self.decode_all(text.rextr(
+                    page, '"text":"', '"', pos))
+            else:
+                user["biography"] = text.unescape(text.remove_html(text.extr(
+                    page, "</span></span></h2>", "<ul>")))
         except Exception:
             if user is None:
                 self.log.debug("Failed to extract user data: %s", data)
@@ -376,16 +386,52 @@ class FacebookExtractor(Extractor):
         return user
 
 
+class FacebookPhotoExtractor(FacebookExtractor):
+    """Base class for Facebook Photo extractors"""
+    subcategory = "photo"
+    pattern = (rf"{BASE_PATTERN}/"
+               rf"(?:[^/?#]+/photos/[^/?#]+/|photo(?:.php)?/?\?"
+               rf"(?:[^&#]+&)*fbid=)([^/?&#]+)[^/?#]*(?<!&setextract)$")
+    example = "https://www.facebook.com/photo/?fbid=PHOTO_ID"
+
+    def items(self):
+        photo_id = self.groups[0]
+        photo_url = f"{self.root}/photo/?fbid={photo_id}&set="
+        photo_page = self.photo_page_request_wrapper(photo_url).text
+
+        i = 1
+        photo = self.parse_photo_page(photo_page)
+        photo["num"] = i
+
+        set_url = f"{self.root}/media/set/?set={photo['set_id']}"
+        set_page = self.request(set_url).text
+
+        directory = self.parse_set_page(set_page)
+
+        yield Message.Directory, "", directory
+        yield Message.Url, photo["url"], photo
+
+        if self.author_followups:
+            for comment_photo_id in photo["followups_ids"]:
+                comment_photo = self.parse_photo_page(
+                    self.photo_page_request_wrapper(
+                        f"{self.root}/photo/?fbid={comment_photo_id}&set="
+                    ).text
+                )
+                i += 1
+                comment_photo["num"] = i
+                yield Message.Url, comment_photo["url"], comment_photo
+
+
 class FacebookSetExtractor(FacebookExtractor):
     """Base class for Facebook Set extractors"""
     subcategory = "set"
     pattern = (
-        BASE_PATTERN +
-        r"/(?:(?:media/set|photo)/?\?(?:[^&#]+&)*set=([^&#]+)"
-        r"[^/?#]*(?<!&setextract)$"
-        r"|([^/?#]+/posts/[^/?#]+)"
-        r"|photo/\?(?:[^&#]+&)*fbid=([^/?&#]+)&set=([^/?&#]+)&setextract)"
-    )
+        rf"{BASE_PATTERN}/"
+        rf"(?:(?:media/set|photo)/?\?(?:[^&#]+&)*set=([^&#]+)"
+        rf"[^/?#]*(?<!&setextract)$"
+        rf"|([^/?#]+/posts/[^/?#]+)"
+        rf"|photo/\?(?:[^&#]+&)*fbid=([^/?&#]+)&set=([^/?&#]+)&setextract)")
     example = "https://www.facebook.com/media/set/?set=SET_ID"
 
     def items(self):
@@ -404,48 +450,11 @@ class FacebookSetExtractor(FacebookExtractor):
         return self.extract_set(set_data)
 
 
-class FacebookPhotoExtractor(FacebookExtractor):
-    """Base class for Facebook Photo extractors"""
-    subcategory = "photo"
-    pattern = (BASE_PATTERN +
-               r"/(?:[^/?#]+/photos/[^/?#]+/|photo(?:.php)?/?\?"
-               r"(?:[^&#]+&)*fbid=)([^/?&#]+)[^/?#]*(?<!&setextract)$")
-    example = "https://www.facebook.com/photo/?fbid=PHOTO_ID"
-
-    def items(self):
-        photo_id = self.groups[0]
-        photo_url = f"{self.root}/photo/?fbid={photo_id}&set="
-        photo_page = self.photo_page_request_wrapper(photo_url).text
-
-        i = 1
-        photo = self.parse_photo_page(photo_page)
-        photo["num"] = i
-
-        set_url = f"{self.root}/media/set/?set={photo['set_id']}"
-        set_page = self.request(set_url).text
-
-        directory = self.parse_set_page(set_page)
-
-        yield Message.Directory, directory
-        yield Message.Url, photo["url"], photo
-
-        if self.author_followups:
-            for comment_photo_id in photo["followups_ids"]:
-                comment_photo = self.parse_photo_page(
-                    self.photo_page_request_wrapper(
-                        f"{self.root}/photo/?fbid={comment_photo_id}&set="
-                    ).text
-                )
-                i += 1
-                comment_photo["num"] = i
-                yield Message.Url, comment_photo["url"], comment_photo
-
-
 class FacebookVideoExtractor(FacebookExtractor):
     """Base class for Facebook Video extractors"""
     subcategory = "video"
     directory_fmt = ("{category}", "{username}", "{subcategory}")
-    pattern = BASE_PATTERN + r"/(?:[^/?#]+/videos/|watch/?\?v=)([^/?&#]+)"
+    pattern = rf"{BASE_PATTERN}/(?:[^/?#]+/videos/|watch/?\?v=)([^/?&#]+)"
     example = "https://www.facebook.com/watch/?v=VIDEO_ID"
 
     def items(self):
@@ -458,7 +467,7 @@ class FacebookVideoExtractor(FacebookExtractor):
         if "url" not in video:
             return
 
-        yield Message.Directory, video
+        yield Message.Directory, "", video
 
         if self.videos == "ytdl":
             yield Message.Url, "ytdl:" + video_url, video
@@ -472,18 +481,18 @@ class FacebookInfoExtractor(FacebookExtractor):
     """Extractor for Facebook Profile data"""
     subcategory = "info"
     directory_fmt = ("{category}", "{username}")
-    pattern = USER_PATTERN + r"/info"
+    pattern = rf"{USER_PATTERN}/info"
     example = "https://www.facebook.com/USERNAME/info"
 
     def items(self):
         user = self._extract_profile(self.groups[0])
-        return iter(((Message.Directory, user),))
+        return iter(((Message.Directory, "", user),))
 
 
 class FacebookAlbumsExtractor(FacebookExtractor):
     """Extractor for Facebook Profile albums"""
     subcategory = "albums"
-    pattern = USER_PATTERN + r"/photos_albums(?:/([^/?#]+))?"
+    pattern = rf"{USER_PATTERN}/photos_albums(?:/([^/?#]+))?"
     example = "https://www.facebook.com/USERNAME/photos_albums"
 
     def items(self):
@@ -516,7 +525,7 @@ class FacebookAlbumsExtractor(FacebookExtractor):
 class FacebookPhotosExtractor(FacebookExtractor):
     """Extractor for Facebook Profile Photos"""
     subcategory = "photos"
-    pattern = USER_PATTERN + r"/photos(?:_by)?"
+    pattern = rf"{USER_PATTERN}/photos(?:_by)?"
     example = "https://www.facebook.com/USERNAME/photos"
 
     def items(self):
@@ -533,7 +542,7 @@ class FacebookPhotosExtractor(FacebookExtractor):
 class FacebookAvatarExtractor(FacebookExtractor):
     """Extractor for Facebook Profile Avatars"""
     subcategory = "avatar"
-    pattern = USER_PATTERN + r"/avatar"
+    pattern = rf"{USER_PATTERN}/avatar"
     example = "https://www.facebook.com/USERNAME/avatar"
 
     def items(self):
@@ -549,13 +558,13 @@ class FacebookAvatarExtractor(FacebookExtractor):
         set_page = self.request(set_url).text
         directory = self.parse_set_page(set_page)
 
-        yield Message.Directory, directory
+        yield Message.Directory, "", directory
         yield Message.Url, avatar["url"], avatar
 
 
 class FacebookUserExtractor(Dispatch, FacebookExtractor):
     """Extractor for Facebook Profiles"""
-    pattern = USER_PATTERN + r"/?(?:$|\?|#)"
+    pattern = rf"{USER_PATTERN}/?(?:$|\?|#)"
     example = "https://www.facebook.com/USERNAME"
 
     def items(self):
